import { Component, OnInit, ViewChild } from "@angular/core";
import { ActivatedRoute, ParamMap } from "@angular/router";
import { OverlayScrollbarsComponent } from "overlayscrollbars-ngx";
import { MainService } from "../../services/main.service";
import { ToolService } from "../../services/tool.service";

@Component({
  selector: "app-data-building",
  templateUrl: "./data-building.component.html",
  styleUrls: ["./data-building.component.less"],
})
export class DataBuildingComponent implements OnInit {
  @ViewChild("osComponentRef1", {
    read: OverlayScrollbarsComponent,
    static: false,
  })
  osComponentRef1: OverlayScrollbarsComponent;
  hosId: any; //院区id
  loading1 = false; //获取楼栋信息列表加载状态
  loading3 = false; //保存
  id: number = 0; //自定义id，负整数
  buildingList: Array<any> = []; //楼栋信息列表
  promptContent: string; //操作提示框提示信息
  ifSuccess: boolean; //操作成功/失败
  promptInfo: string; //操作结果提示信息
  promptModalShow: boolean; //操作提示框是否展示
  delModal: boolean = false; //删除模态框
  tipsMsg1: string; //提示框信息
  tips: string; //提示框信息(附注)
  coop: any; //当前操作列
  loading2 = false; //删除接口
  mainWidth: any = "100%";
  coopBtns = {};
  constructor(
    private mainService: MainService,
    private route: ActivatedRoute,
    private tool: ToolService
  ) {}
  ngOnInit() {
    this.route.queryParamMap.subscribe((params: ParamMap) => {
      this.hosId = params.get("id");
      if (!this.hosId) {
        this.hosId = this.tool.getCurrentHospital().id;
      }
      this.getBuildingList();
    });
    this.coopBtns = this.tool.initCoopBtns("dataDictionary");
    this.changeSize();
    window.addEventListener("resize", () => {
      this.changeSize();
    });
  }
  // 改变窗口大小
  changeSize() {
    let w = window.innerWidth - 582; //屏幕宽-其他元素宽度
    this.mainWidth = Math.floor(w / 360) * 360 + "px";
  }
  // 新增
  add() {
    this.buildingList.push({ id: --this.id, buildingName: "" });
  }
  // 删除
  delete(data) {
    this.showDelModal(
      data,
      "您确认要删除此楼栋吗？",
      "删除后对应的楼栋积分配置也会删除"
    );
  }
  // 删除楼栋
  showDelModal(data: any, tipsMsg1: string, tips?) {
    this.delModal = true;
    this.coop = data;
    this.tipsMsg1 = tipsMsg1;
    this.tips = tips ? tips : "";
  }
  // 隐藏删除框
  hideDelModal() {
    this.delModal = false;
  }
  // 确认删除
  confirmDel() {
    this.loading2 = true;
    if (this.coop.id > 0) {
      //接口删除
      let postData = [this.coop.id];
      this.mainService.delBuildingList(postData).subscribe(
        (result) => {
          this.loading2 = false;
          this.delModal = false;
          if (result["status"] == 200 && !result["data"][0].msg) {
            this.showPromptModal("删除", true, "", false);
            // 过滤掉当前这一条
            this.buildingList = this.buildingList.filter(
              (item) => item.id !== this.coop.id
            );
          } else {
            this.showPromptModal("删除", false, result["data"][0].msg, false);
          }
        },
        (err) => {
          this.loading2 = false;
          this.delModal = false;
          this.showPromptModal("删除", false, "", false);
        }
      );
    } else {
      this.loading2 = false;
      this.delModal = false;
      this.showPromptModal("删除", true, "", false);
      // 过滤掉当前这一条
      this.buildingList = this.buildingList.filter(
        (item) => item.id !== this.coop.id
      );
    }
  }
  // 保存
  save() {
    let arr = []; //排查是否有重复的名称的容器
    let arrStr = ""; //重复名称
    // 过滤空值
    this.buildingList = this.buildingList.filter((item) => {
      return item.buildingName !== "";
    });
    // 数据处理
    let postData = JSON.parse(JSON.stringify(this.buildingList));
    postData.forEach((item) => {
      arr.push(item.buildingName);
      item.deleted = false;
      item.hosId = this.hosId;
      if (item.id < 0) {
        delete item.id;
      }
    });
    if (arr.length > 1) {
      arr.sort();
      for (let i = 0; i < arr.length - 1; i++) {
        if (arr[i] === arr[i + 1]) {
          arrStr += `【${arr[i]}】,`;
        }
      }
    }
    if (arrStr) {
      //有重复名称
      arrStr = [...new Set(arrStr.split(","))].join(); //去重
      this.showPromptModal(
        "保存",
        false,
        `存在重复的楼栋名称${arrStr}请修改后再保存！`,
        false
      );
    } else {
      this.loading3 = true;
      this.mainService.saveBuildingList(postData).subscribe(
        (result) => {
          this.loading3 = false;
          if (result["status"] == 200) {
            this.showPromptModal("保存", true, "");
          } else {
            this.showPromptModal("保存", false, result["msg"]);
          }
        },
        (err) => {
          this.loading3 = false;
          this.showPromptModal("保存", false, "");
        }
      );
    }
  }
  // 展示信息提示框（con:提示信息，success:操作是否成功，promptInfo:操作结果提示信息）
  showPromptModal(con, success, promptInfo, isRefresh = true) {
    this.promptModalShow = false;
    this.promptContent = con;
    this.ifSuccess = success;
    this.promptInfo = promptInfo;
    setTimeout(() => {
      this.promptModalShow = true;
    }, 100);
    if (isRefresh) {
      this.getBuildingList();
    }
  }
  // 获取楼栋信息列表
  getBuildingList() {
    this.loading1 = true;
    this.mainService
      .getFetchDataList("simple/data", "building", {
        idx: 0,
        sum: 9999,
        building: { hosId: this.hosId },
      })
      .subscribe((result) => {
        this.loading1 = false;
        if (result["status"] == 200) {
          if (
            result["list"] &&
            Array.isArray(result["list"]) &&
            result["list"].length < 6
          ) {
            let arr = result["list"];
            let len = 6 - arr.length;
            for (let i = 0; i < len; i++) {
              arr.push({ id: --this.id, buildingName: "" });
            }
            this.buildingList = arr;
          } else if (
            result["list"] &&
            Array.isArray(result["list"]) &&
            result["list"].length >= 6
          ) {
            this.buildingList = result["list"];
          } else {
            let arr = [];
            for (let i = 0; i < 6; i++) {
              arr.push({ id: --this.id, buildingName: "" });
            }
            this.buildingList = arr;
          }
        }
      });
  }
}
