import { Component, OnInit } from "@angular/core";
import { MainService } from 'src/app/services/main.service';
import { ToolService } from 'src/app/services/tool.service';
import { NzMessageService } from 'ng-zorro-antd';

@Component({
  selector: "app-inspect-and-patient-transport-config",
  templateUrl: "./inspect-and-patient-transport-config.component.html",
  styleUrls: ["./inspect-and-patient-transport-config.component.less"],
})
export class InspectAndPatientTransportConfigComponent implements OnInit {
  loading:boolean = false; //页面加载的loading
  btnLoading:boolean = false; //提交按钮的loading
  tabModalName:string = 'characteristics'; //当前选中的tab
  hosId = this.tool.getCurrentHospital().id; //当前院区
  // 检查预约时间是否精准填写-时间模式
  timeModes:any[] = [
    {id: 1, name: '每分钟'},
    {id: 10, name: '每10分钟'},
    {id: 30, name: '每30分钟'},
  ];
  // 是否开通送回病房
  accurateTimeFilling:any[] = [
    {label:'是否开启',value: 0}
  ];
  // 是否显示排队信息
  queuingInformation:any[] = [
    {label:'是否开启',value: 0}
  ];
  timeMod:any;
  // 交接方式
  // handoverMode:any;
  // handoverModes:any[] = [];
  // 核对方式
  // checkModes:any[] = [];
  // 签到方式
  checkInModes:any[] = [];
  // 多人模式支持
  multiplayerMode:any[] = [
    {label:'是否开启',value: 0}
  ];
  // 积分计算方式
  integralCalculationMethod:any = null;
  // 自动关单
  // automaticCustomsOrders:any[] = [];
  // 自动建单
  // autoCreateOrders:any[] = [
  //   {label:'是否开启',value: 0}
  // ];
  // 是否根据开通科室建单
  // autoDepts:any[] = [
  //   {label:'是否根据开通科室建单',value: 0}
  // ];
  // 配置
  configs:any = {};
  // 任务类型
  tasktype:any = {};
  constructor(private mainService: MainService,private tool: ToolService,private msg: NzMessageService) {}

  ngOnInit():void {
    // todo
    // this.getHandoverModes();
    // this.getCheckModes();
    this.getCheckInModes();
  }

  // 修改是否多人模式支持
  changeMultiplayerMode(e){
    console.log(e);
    if(!e[0].checked){
      this.integralCalculationMethod = null;
    }
  }
  // 修改检查预约时间是否精准填写
  changeAccurateTimeFilling(e){
    console.log(e);
    if(!e[0].checked){
      // todo
      this.timeMod = null;
    }
  }
  //获取积分计算方式
  integralCalculationMethods:any = [];
  dLoading = false;
  getIntegralCalculationMethod() {
    this.dLoading = true;
    this.mainService
      .getDictionary("list", "integral_calculation_method")
      .subscribe((data) => {
        this.dLoading = false;
        this.integralCalculationMethods = data;
        this.getTaskType();
      });
  }
  // 切换tab
  tabModal(tabModalName:string){
    this.tabModalName = tabModalName;
  }
  // 修改是否自动建单
  // changeAutoCreateOrders(e){
  //   console.log(e);
  //   if(!e[0].checked){
  //     this.autoDepts[0].checked = false;
  //   }
  // }
  // 保存
  submitForm() {
    if(!this.tasktype.id){
      this.msg.create("warning", "请先配置患者陪检任务类型！");
      return;
    }
    if(this.multiplayerMode[0].checked && !this.integralCalculationMethod){
      this.msg.create("warning", "请选择积分计算方式！");
      return;
    }
    if(this.accurateTimeFilling[0].checked && !this.timeMod){
      this.msg.create("warning", "请选择精确时间模式！");
      return;
    }
    // if(!this.handoverMode){
    //   this.msg.create("warning", "请选择交接方式！");
    //   return;
    // }
    // if(!this.radioCheckModesValue){
    //   this.msg.create("warning", "请选择核对方式！");
    //   return;
    // }
    let signTypeIds = this.checkInModes.filter(v => v.checked).map(v => v.value).toString();
    // let closeTypeIds = this.automaticCustomsOrders.filter(v => v.checked).map(v => v.value).toString();
    let postData:any = {
      id: this.configs.id,
      taskType: this.tasktype.id,
      hosId: this.hosId,
      multiplayerMode: this.multiplayerMode[0].checked ? 1 : 0,
      integralCalculationMethod: this.multiplayerMode[0].checked ? this.integralCalculationMethod : undefined,
      accurateTimeFilling: this.accurateTimeFilling[0].checked ? 1 : 0,
      queuingInformation: this.queuingInformation[0].checked ? 1 : 0,
      timeMod: this.timeMod || undefined,
      // autoCreate: this.autoCreateOrders[0].checked ? 1 : 0,
      // autoDept: this.autoDepts[0].checked ? 1 : 0,
      // handoverType: {id: this.handoverMode},
      // checkType: {id: this.radioCheckModesValue},
      signTypeIds: signTypeIds || undefined,
      // closeTypeIds: closeTypeIds || undefined,
    };
    this.btnLoading = true;
    this.mainService
      .simplePost("addData", "taskTypeConfig", postData)
      .subscribe((result) => {
        this.btnLoading = false;
        if (result.status == 200) {
          this.getConfig();
        }
      });
  }

  //获取交接方式
  // getHandoverModes() {
  //   this.mainService.getDictionary("list", "ttconfig_handover_type").subscribe((data) => {
  //     this.handoverModes = data;
  //     this.handoverMode = data[0].id;
  //   });
  // }
  //获取核对方式
  // getCheckModes() {
  //   this.mainService.getDictionary("list", "ttconfig_check_type").subscribe((data) => {
  //     this.checkModes = data.map(v => ({label:v.name, value: v.id}));
  //   });
  // }
  //获取签到方式
  getCheckInModes() {
    this.loading = true;
    this.mainService.getDictionary("list", "inspect_config_sign_type").subscribe((data) => {
      this.checkInModes = data.map(v => ({label:v.name, value: v.id}));
      // this.getAutomaticCustomsOrders();
      // this.getTaskType();
      this.getIntegralCalculationMethod();
    });
  }
  //获取自动关单
  // getAutomaticCustomsOrders() {
  //   this.mainService.getDictionary("list", "ttconfig_close_type").subscribe((data) => {
  //     this.automaticCustomsOrders = data.map(v => ({label:v.name, value: v.id}));
  //     this.getTaskType();
  //   });
  // }
  //获取任务类型
  getTaskType() {
    let postData = {
      idx: 0,
      sum: 10,
      taskType: {
        simpleQuery: true,
        hosId: {
          id: this.hosId
        },
        associationType: {
          key:"association_types",
          value: 'inspect'
        }
      }
    };
    this.mainService
      .getFetchDataList("simple/data", "taskType", postData)
      .subscribe((result) => {
        if (result.status == 200) {
          this.tasktype = result.list[0] || {};
          this.getConfig();
        }
      });
  }
  // 获取配置
  // radioCheckModesValue = '';
  getConfig() {
    this.loading = true;
    let postData = {
      idx: 0,
      sum: 10,
      taskTypeConfig: {
        taskTypeDTO: {
          hosId: {
            id: this.hosId
          },
          associationType: this.tasktype.associationType
        }
      }
    };
    this.mainService
      .getFetchDataList("simple/data", "taskTypeConfig", postData)
      .subscribe((result) => {
        this.loading = false;
        if (result.status == 200) {
          this.configs = result.list[0] || {};
          // this.handoverMode = this.configs.handoverType?this.configs.handoverType.id:undefined;
          // this.radioCheckModesValue = this.configs.checkType.id;
          // this.autoCreateOrders[0].checked = this.configs.autoCreate == 1;
          // this.autoDepts[0].checked = this.configs.autoDept == 1;

          if(this.configs.signTypeIds){
            let ids = this.configs.signTypeIds.split(',');
            this.checkInModes = this.checkInModes.map(v => ({...v, checked: ids.includes(v.value.toString())}));
          }
          this.multiplayerMode[0].checked = this.configs.multiplayerMode == 1;
          this.integralCalculationMethod = this.configs.integralCalculationMethod;
          this.accurateTimeFilling[0].checked = this.configs.accurateTimeFilling == 1;
          this.timeMod = this.configs.timeMod || null;
          this.queuingInformation[0].checked = this.configs.queuingInformation == 1;


          // if(this.configs.closeTypeIds){
          //   let ids = this.configs.closeTypeIds.split(',');
          //   this.automaticCustomsOrders = this.automaticCustomsOrders.map(v => ({...v, checked: ids.includes(v.value.toString())}));
          // }
        }
      });
  }
}


